Generate warnings in places where we want to deprecate functions or struct parameters
Summary
Functions
Generates a deprecation warning for using fields[relationship_key] instead of fields[type] when
parsing query parameters.
Generate warnings in places where we want to deprecate functions or struct parameters
Generates a deprecation warning for using fields[relationship_key] instead of fields[type] when
parsing query parameters.